@@ -9,6 +9,7 @@
 #include "json_p.h"
 #include "datatypes_p.h"
 #include "loadutil_p.h"
+#include "logging.h"
 #include "mergeutil_p.h"
 #include "notesutil_p.h"
 #include "platformutils_p.h"
@@ -118,10 +119,10 @@ int JourneySection::distance() const
         return 0;
     }
 
-    int dist = 0;
+    double dist = 0;
     if (d->from.hasCoordinate() && d->to.hasCoordinate()) {
-        float startLat = d->from.latitude();
-        float startLon = d->from.longitude();
+        auto startLat = d->from.latitude();
+        auto  startLon = d->from.longitude();
 
         for (const auto &stop : d->intermediateStops) {
             if (!stop.stopPoint().hasCoordinate()) {
@@ -134,8 +135,8 @@ int JourneySection::distance() const
 
         dist += Location::distance(startLat, startLon, d->to.latitude(), d->to.longitude());
     }
-    dist = std::max(dist, d->path.distance());
-    return std::max(dist, d->distance);
+    dist = std::max<double>(dist, d->path.distance());
+    return std::max((int)std::round(dist), d->distance);
 }
 
 void JourneySection::setDistance(int value)
@@ -303,38 +304,97 @@ void JourneySection::setArrival(const Stopover &arrival)
     setArrivalVehicleLayout(arrival.vehicleLayout());
 }
 
+struct {
+    Line::Mode mode;
+    int gramPerKm;
+} static constexpr const emissionForModeMap[] = {
+    { Line::Air, 285 },
+    { Line::Boat, 245 },
+    { Line::Bus, 68 },
+    { Line::Coach, 68 },
+    { Line::Ferry, 245 },
+    // { Line::Funicular, -1 }, TODO
+    { Line::LocalTrain, 14 },
+    { Line::LongDistanceTrain, 14 },
+    { Line::Metro, 11 },
+    { Line::RailShuttle, 11 }, // assuming tram/rapid transit-like
+    { Line::RapidTransit, 11 },
+    { Line::Shuttle, 68 },
+    { Line::Taxi, 158 },
+    { Line::Train, 14 },
+    { Line::Tramway, 11 },
+    { Line::RideShare, 158 },
+    // { Line::AerialLift, -1 }, TODO
+};
+
+struct {
+    IndividualTransport::Mode mode;
+    int gramPerKm;
+} static constexpr const emissionForIvModeMap[] = {
+    { IndividualTransport::Walk, 0 },
+    { IndividualTransport::Bike, 0 },
+    { IndividualTransport::Car, 158 }
+};
+
+struct {
+    RentalVehicle::VehicleType mode;
+    int gramPerKm;
+} static constexpr const emissionForRvModeMap[] = {
+    { RentalVehicle::Bicycle, 0 },
+    // { RentalVehicle::Pedelec, -1 }, TODO
+    // { RentalVehicle::ElectricKickScooter, -1 }, TODO
+    // { RentalVehicle::ElectricMoped, -1 }, TODO
+    { RentalVehicle::Car, 158 },
+};
+
 int JourneySection::co2Emission() const
 {
-    // TODO handle rental vehicles and ride sharing in here!
     if (d->co2Emission >= 0) {
         return d->co2Emission;
     }
 
-    struct {
-        Line::Mode mode;
-        int gramPerKm;
-    } static const emissionForModeMap[] = {
-        { Line::Air, 285 },
-        { Line::Boat, 245 },
-        { Line::Bus, 68 },
-        { Line::BusRapidTransit, 68 },
-        { Line::Coach, 68 },
-        { Line::Ferry, 245 },
-        { Line::LocalTrain, 14 },
-        { Line::LongDistanceTrain, 14 },
-        { Line::Metro, 11 },
-        { Line::RapidTransit, 11 },
-        { Line::Taxi, 158 },
-        { Line::Train, 14 },
-        { Line::Tramway, 11 },
-    };
-
-    const auto mode = route().line().mode();
-    for (const auto &map : emissionForModeMap) {
-        if (map.mode == mode) {
-            return (map.gramPerKm * distance()) / 1000;
+    switch (d->mode) {
+        case JourneySection::Invalid:
+            return -1;
+        case JourneySection::Walking:
+        case JourneySection::Transfer:
+        case JourneySection::Waiting:
+            return 0;
+        case JourneySection::PublicTransport:
+        {
+            const auto mode = route().line().mode();
+            for (const auto &map : emissionForModeMap) {
+                if (map.mode == mode) {
+                    return (map.gramPerKm * distance()) / 1000;
+                }
+            }
+            qCDebug(Log) << "No CO2 emission estimate for mode" << mode;
+            return -1;
+        }
+        case JourneySection::IndividualTransport:
+        {
+            const auto mode = individualTransport().mode();
+            for (const auto &map :emissionForIvModeMap) {
+                if (map.mode == mode) {
+                    return (map.gramPerKm *distance()) / 1000;
+                }
+            }
+            qCDebug(Log) << "No CO2 emission estimate for mode" << mode;
+            return -1;
+        }
+        case JourneySection::RentedVehicle:
+        {
+            const auto mode = rentalVehicle().type();
+            for (const auto &map :emissionForRvModeMap) {
+                if (map.mode == mode) {
+                    return (map.gramPerKm *distance()) / 1000;
+                }
+            }
+            qCDebug(Log) << "No CO2 emission estimate for vehicle type" << mode;
+            return -1;
         }
     }
+
     return -1;
 }
 
@@ -392,16 +452,36 @@ QString JourneySection::iconName() const
             return {};
         case JourneySection::PublicTransport:
             return d->route.line().iconName();
-        case JourneySection::Transfer:
-            return u"qrc:///org.kde.kpublictransport/assets/images/transfer.svg"_s;
-        case JourneySection::Walking:
-            return IndividualTransport::modeIconName(IndividualTransport::Walk);
-        case JourneySection::Waiting:
-            return u"qrc:///org.kde.kpublictransport/assets/images/wait.svg"_s;
         case JourneySection::RentedVehicle:
             return d->rentalVehicle.vehicleTypeIconName();
         case JourneySection::IndividualTransport:
             return d->individualTransport.modeIconName();
+        case JourneySection::Transfer:
+        case JourneySection::Walking:
+        case JourneySection::Waiting:
+            break;
+    }
+
+    return modeIconName(d->mode);
+}
+
+QString JourneySection::modeIconName(JourneySection::Mode mode)
+{
+    switch (mode) {
+        case JourneySection::Invalid:
+            return {};
+        case JourneySection::PublicTransport:
+            return Line::modeIconName(Line::Train);
+        case JourneySection::Transfer:
+            return u"qrc:///org.kde.kpublictransport/assets/images/journey-mode-transfer.svg"_s;
+        case JourneySection::Walking:
+            return IndividualTransport::modeIconName(IndividualTransport::Walk);
+        case JourneySection::Waiting:
+            return u"qrc:///org.kde.kpublictransport/assets/images/journey-mode-wait.svg"_s;
+        case JourneySection::RentedVehicle:
+            return RentalVehicle::vehicleTypeIconName(RentalVehicle::ElectricKickScooter);
+        case JourneySection::IndividualTransport:
+            return IndividualTransport::modeIconName(IndividualTransport::Bike);
     }
 
     return u"question"_s;
@@ -696,6 +776,16 @@ Disruption::Effect Journey::disruptionEffect() const
     return effect;
 }
 
+int Journey::distance() const
+{
+    return std::accumulate(d->sections.begin(), d->sections.end(), 0, [](auto dist, const auto &jny) { return dist + jny.distance(); });
+}
+
+int Journey::co2Emission() const
+{
+    return std::accumulate(d->sections.begin(), d->sections.end(), 0, [](auto co2, const auto &jny) { return co2 + std::max(0, jny.co2Emission()); });
+}
+
 void Journey::applyMetaData(bool download)
 {
     for (auto &sec : d->sections) {
